/* 
 * Copyright 2023-2025 Licensed under the AGPL License
 */
package test.mysql;

import com.flowlong.bpm.engine.QueryService;
import com.flowlong.bpm.engine.TaskService;
import com.flowlong.bpm.engine.entity.FlwHisTask;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 测试简单流程
 *
 * @author ming
 */
@Slf4j
class TestSimpleProcess extends MysqlTest {

    @BeforeEach
    public void before() {
        processId = this.deployByResource("test/simpleProcess.json", testCreator);
    }

    @Test
    void test() {
        // 启动指定流程定义ID启动流程实例
        Map<String, Object> args = new HashMap<>();
        args.put("day", 8);
        args.put("age", 18);
        args.put("assignee", testUser1);

        // 启动指定流程定义ID启动流程实例
        flowLongEngine.startInstanceById(processId, testCreator, args).ifPresent(instance -> {

            // 测试会签审批人001【审批】
            this.executeTask(instance.getId(), testCreator);

            // 延迟下一会签任务完成时间
            try {
                Thread.sleep(2000);
            } catch (InterruptedException e) {
                // to do noting
            }

            // 测试会签审批人003【审批】
            this.executeActiveTasks(instance.getId(), test3Creator, args);

            //撤回任务(条件路由子审批) 回到测试会签审批人003【审批】任务
            QueryService queryService = flowLongEngine.queryService();
            List<FlwHisTask> hisTasks = queryService.getHisTasksByInstanceId(instance.getId()).get();
            FlwHisTask hisTask = hisTasks.get(0);
            TaskService taskService = flowLongEngine.taskService();
            taskService.withdrawTask(hisTask.getId(), testCreator);

            // 测试会签审批人003【审批】
            this.executeActiveTasks(instance.getId(), test3Creator, args);

            // 年龄审批【审批】
            this.executeTask(instance.getId(), testCreator);

            // 条件内部审核【审批】
            this.executeTask(instance.getId(), testCreator);

            // 条件路由子审批【审批】 抄送 结束
            this.executeTask(instance.getId(), testCreator);

        });
    }
}
